import 'package:flutter/material.dart';

class AnimatedGradient extends StatefulWidget {
  final List<Color> colors;
  final Widget? child;
  AnimatedGradient({Key? key, required this.colors, this.child}) : super(key: key){
    if(colors.length < 2){
      throw Exception("length needs to be at least 2");
    }
  }

  @override
  _AnimatedGradientState createState() => _AnimatedGradientState();
}

class _AnimatedGradientState extends State<AnimatedGradient> {
  List<Alignment> alignmentList = [
    Alignment.bottomLeft,
    Alignment.bottomRight,
    Alignment.topRight,
    Alignment.topLeft,
  ];
  int index = 0;
  late Color bottomColor;
  late Color topColor;
  Alignment begin = Alignment.bottomLeft;
  Alignment end = Alignment.topRight;

  @override
  void initState() {
    bottomColor = widget.colors[0];
    topColor = widget.colors[1];
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Stack(
          children: [
            AnimatedContainer(
              duration: const Duration(seconds: 2),
              onEnd: () {
                setState(() {
                  index = index + 1;
                  // animate the color
                  bottomColor = widget.colors[index % widget.colors.length];
                  topColor = widget.colors[(index + 1) % widget.colors.length];

                  //// animate the alignment
                  // begin = alignmentList[index % alignmentList.length];
                  // end = alignmentList[(index + 2) % alignmentList.length];
                });
              },
              decoration: BoxDecoration(
                  gradient: LinearGradient(
                      begin: begin, end: end, colors: [bottomColor, topColor])),
            ),
            widget.child ?? const SizedBox(),
          ],
        ));
  }
}
