import 'package:flutter/material.dart';
import 'package:progress_state_button/iconed_button.dart';
import 'package:progress_state_button/progress_button.dart';
import '../constants.dart';

typedef FutureRequest = Future<dynamic> Function();
typedef Validator = bool Function();

class DefaultLoadingButton extends StatefulWidget {
  final Color? backgroundColor;
  final IconData icon;
  final String title;
  final String? failText;
  final String? loadingText;
  final String? successText;
  final FutureRequest request;
  final Function({dynamic value}) onSuccess;
  final Function onFailure;
  final Validator? validator;
  final double? width;
  final double? height;

  const DefaultLoadingButton({Key? key,
    this.width,
    this.height,
    required this.request,
    required this.onSuccess,
    required this.onFailure,
    required this.backgroundColor,
    required this.icon,
    required this.title,
    this.validator,
    this.failText,
    this.loadingText,
    this.successText})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _DefaultLoadingButtonState();
  }
}

class _DefaultLoadingButtonState extends State<DefaultLoadingButton>{
  ButtonState buttonState = ButtonState.idle;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.width ?? 150,
      height: widget.height ?? 50,
      child: ProgressButton.icon(
        iconedButtons: {
          ButtonState.idle: IconedButton(
              text: widget.title,
              icon: Icon(widget.icon, color: Colors.white),
              color: widget.backgroundColor ?? Colors.black),
          ButtonState.loading: IconedButton(
              text: widget.loadingText ?? "Loading", color: defaultTitleColor),
          ButtonState.fail: IconedButton(
              text: widget.failText ?? "Failed",
              icon: const Icon(Icons.cancel, color: Colors.white),
              color: defaultDangerColor),
          ButtonState.success: IconedButton(
              text: widget.successText ?? "Success",
              icon: const Icon(
                Icons.check_circle,
                color: Colors.white,
              ),
              color: defaultSuccessColor)
        },
        onPressed: () {
          if (buttonState == ButtonState.idle && (widget.validator == null || widget.validator!() == true)) {
            setState(() {
              buttonState = ButtonState.loading;
            });
            widget.request().catchError((onError) {
              setState(() {
                buttonState = ButtonState.fail;
              });
              widget.onFailure();
              Future.delayed(const Duration(milliseconds: 2000), () {
                setState(() {
                  buttonState = ButtonState.idle;
                });
              });
            }).then((value) {
              setState(() {
                buttonState = ButtonState.success;
              });
              widget.onSuccess(value: value);
              Future.delayed(const Duration(milliseconds: 2000), () {
                setState(() {
                  buttonState = ButtonState.idle;
                });
              });
            });
          }
        },
        state: buttonState,
        radius: defaultCornerRadius,
      ),
    );
  }
}
